@@ -13,7 +13,7 @@ import tui
 TASK_FIELDS = ["title", "creationDate", "dueDate", "doneDate", "description", "urgency", "status", "project", "keywords"]
 
 class CsvListRenderer(object):
-    def __init__(self, out):
+    def __init__(self, out, cryptoMgr=None):
         self.writer = csv.writer(out, dialect="excel")
         self._writerow(TASK_FIELDS) # Header
 
