@@ -22,16 +22,13 @@ class YokadiOptionParser(OptionParser):
         OptionParser.__init__(self)
 
     def parse_args(self, line):
-        nargv = [] # New argv with escaped arg if needed or keyword switch change
-        earg = []  # Escaped argument
-
         argv = line.split(u" ")
         # Splitting an empty line gives us [""], not an empty array
         if argv == [u""]:
             argv = []
 
         # Unknown options will throw an error
-        options, args =  OptionParser.parse_args(self, argv)
+        options, args = OptionParser.parse_args(self, argv)
         return options, args
 
 
